@@ -1,7 +1,7 @@
 # ABSTRACT: Utilities for Monitoring ElasticSearch
 package App::ElasticSearch::Utilities;
 
-our $VERSION = '4.0'; # VERSION
+our $VERSION = '4.1'; # VERSION
 
 use strict;
 use warnings;
@@ -15,12 +15,14 @@ our @_CONFIGS = (
     "$ENV{HOME}/.es-utils.yml",
 );
 
-use CLI::Helpers qw(:all);
-use Time::Local;
-use Getopt::Long qw(:config pass_through);
-use JSON::XS;
-use YAML;
 use Elastijk;
+use CLI::Helpers qw(:all);
+use Getopt::Long qw(:config pass_through);
+use Hash::Merge::Simple qw(clone_merge);
+use JSON::XS;
+use Time::Local;
+use YAML;
+
 use Sub::Exporter -setup => {
     exports => [ qw(
         es_globals
@@ -47,6 +49,7 @@ use Sub::Exporter -setup => {
         es_apply_index_settings
     )],
     groups => {
+        config  => [qw(es_globals)],
         default => [qw(es_connect es_indices es_request)],
         indices => [qw(:default es_indices_meta)],
         index   => [qw(:default es_index_valid es_index_fields es_index_days_old)],
@@ -72,18 +75,21 @@ if( !defined $_OPTIONS_PARSED ) {
     );
     $_OPTIONS_PARSED = 1;
 }
+my @ConfigData=();
 foreach my $config_file (@_CONFIGS) {
     next unless -f $config_file;
     debug("Loading options from $config_file");
-    my %from_file = ();
     eval {
         my $ref = YAML::LoadFile($config_file);
-        %from_file = %{$ref};
+        push @ConfigData, $ref;
         debug_var($ref);
+        1;
+    } or do {
+        debug({color=>"red"}, "[$config_file] $@");
     };
-    debug({color=>"red"}, "[$config_file] $@") if $@;
-    $_GLOBALS{$_} = $from_file{$_} for keys %from_file;
 }
+%_GLOBALS = %{ clone_merge(@ConfigData) };
+
 # Set defaults
 my %DEF = (
     # Connection Options
@@ -168,7 +174,10 @@ sub _get_es_version {
             $CURRENT_VERSION = join('.', (split /\./,$result->{version}{number})[0,1]);
         }
     };
-    $CURRENT_VERSION ||= 0;
+    if( !defined $CURRENT_VERSION || $CURRENT_VERSION <= 0 ) {
+        output({color=>'red',stderr=>1}, "Unable to determine Elasticsearch version, something has gone terribly wrong: aborting.");
+        exit 1;
+    }
     debug({color=>'magenta'}, "FOUND VERISON '$CURRENT_VERSION'");
     return $CURRENT_VERSION;
 };
@@ -604,7 +613,7 @@ App::ElasticSearch::Utilities - Utilities for Monitoring ElasticSearch
 
 =head1 VERSION
 
-version 4.0
+version 4.1
 
 =head1 SYNOPSIS
 
@@ -799,7 +808,7 @@ From App::ElasticSearch::Utilities:
     --pattern       Use a pattern to operate on the indexes
     --days          If using a pattern or base, how many days back to go, default: all
 
-=head2 ARGUMENT GLOBALS
+=head1 ARGUMENT GLOBALS
 
 Some options may be specified in the B</etc/es-utils.yaml> or B<$HOME/.es-utils.yaml> file:
 
@@ -857,11 +866,6 @@ regular expressions.  Those patterns are:
     DATE    expands to match YYYY.MM.DD, YYYY-MM-DD, or YYYYMMDD
     ANY     expands to match any number of any characters.
 
-=head2 CONTRIBUTORS
-
-    Mihai Oprea <mishu@mishulica.com>
-    Samit Badle
-
 =head1 AUTHOR
 
 Brad Lhotsky <brad@divisionbyzero.net>
@@ -874,4 +878,40 @@ This is free software, licensed under:
 
   The (three-clause) BSD License
 
+=head1 CONTRIBUTORS
+
+=for stopwords Alexey Shatlovsky Surikov Daniel Ostermeier Jason Rojas Kang-min Liu Samit Badle Takumi Sakamoto
+
+=over 4
+
+=item *
+
+Alexey Shatlovsky <alexey.shatlovsky@booking.com>
+
+=item *
+
+Alexey Surikov <ksurent@gmail.com>
+
+=item *
+
+Daniel Ostermeier <daniel.ostermeier@gmail.com>
+
+=item *
+
+Jason Rojas <jason.rojas@mgo.com>
+
+=item *
+
+Kang-min Liu <gugod@gugod.org>
+
+=item *
+
+Samit Badle <Samit.Badle@gmail.com>
+
+=item *
+
+Takumi Sakamoto <takumi.saka@gmail.com>
+
+=back
+
 =cut
