@@ -1,7 +1,7 @@
 # ABSTRACT: Fix version issues to support all the things
 package App::ElasticSearch::Utilities::VersionHacks;
 
-our $VERSION = '4.0'; # VERSION
+our $VERSION = '4.1'; # VERSION
 use strict;
 use warnings;
 
@@ -17,7 +17,22 @@ const my $MIN_VERSION => 1.0;
 const my %SIMPLE => (
     '_cluster/nodes' => {
         default => '_nodes',
-    }
+    },
+    '_optimize' => {
+        # Yes, in case you're wondering _optimize disappeared in 2.2 after being deprecated in 2.1
+        default => '_forcemerge',
+        1.0 => '_optimize',
+        1.1 => '_optimize',
+        1.2 => '_optimize',
+        1.3 => '_optimize',
+        1.4 => '_optimize',
+        1.5 => '_optimize',
+        1.6 => '_optimize',
+        1.7 => '_optimize',
+        1.8 => '_optimize',
+        1.9 => '_optimize',
+        2.0 => '_optimize',
+    },
 );
 my %CALLBACKS = (
     '_cluster/state' => {
@@ -142,7 +157,7 @@ App::ElasticSearch::Utilities::VersionHacks - Fix version issues to support all
 
 =head1 VERSION
 
-version 4.0
+version 4.1
 
 =head1 AUTHOR
 
